"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interfaceOfClass = void 0;
const interfaceOfClass = (Klass) => {
    /**
     * Get properties of a class
     * @see https://stackoverflow.com/a/40637896/1123955
     */
    /**
     * Huan(202110): we decide not to check the instance properties
     *  because the instance properties are not always the same as the class properties
     */
    // const instance = new Klass()
    // const instanceProperties = Object.getOwnPropertyNames(instance)
    const prototypeProperties = Object
        .getOwnPropertyNames(Klass.prototype)
        .filter(property => property !== 'constructor');
    if (prototypeProperties.length === 0) {
        throw new Error(`${Klass.name} has no prototype properties`);
    }
    return () => (target) => {
        if (target instanceof Object) {
            return prototypeProperties
                .every(prop => prop in target);
        }
        return false;
    };
};
exports.interfaceOfClass = interfaceOfClass;
//# sourceMappingURL=interface-of-class.js.map